#!/usr/bin/env python
"""
Parses this:
    sequence(
        resource(name=health, level=0.5),
        dosomething()
    )
Into this:
    "title": "sequence",
    "type": "composite",
    "name": sequence",
    "parameters": {},
    "children": [
      {
        "title": "resource",
        "type": "action",
        "name": resource",
        "parameters": {},
      }
    ]
"""

import sys
import json
from collections import OrderedDict
from collections import namedtuple
from textwrap import dedent

class node(object):
    script = None
    param_order = None
    title = "node"
    ntype = "unkown"
    def __init__(self, *children, **parameters):
        self.children = children
        if 'children' in parameters:
            self.children = parameters.pop('children')
        elif 'child' in parameters:
            self.children = [parameters.pop('child')]
        self.output = parameters.pop('output', None)
        self.parameters = parameters

    def expand_parameters(self):
        parameters = OrderedDict()
        keys = self.parameters.keys()
        try:
            keys = sorted(keys, key=lambda k: self.param_order.index(k))
        except Exception:
            pass
        for k in keys:
            v = self.parameters[k]
            if type(v) != dict:
                parameters[k] = {'value': v}
            else:
                parameters[k] = v
        return parameters

    def prejson(self):
        pass

    def json(self):
        self.prejson()
        json = OrderedDict([
            ("title", self.__class__.title),
            ("type", self.__class__.ntype),
            ("name", self.__class__.name),
            ("parameters", self.expand_parameters()),
            ])
        if self.output:
            json['output'] = self.output
        if len(self.children) == 1:
            json['child'] = self.children[0].json()
        elif len(self.children) > 1:
            json['children'] = list()
            for child in self.children:
                json['children'].append(child.json())
        return json

def make_node(namespace, name, type_, param_order=None):
    if sys.version_info.major == 2:
        if type(name) == unicode:
            name = name.encode('ascii', 'ignore')
    _node = type(name, (node,), dict(
        title=name,
        ntype=type_,
        name=name,
        param_order=param_order))
    namespace[name] = _node
    return _node

def make_composites(namespace):
    for name in dedent("""
            sequence
            selector
            dynamic
            randomize"""[1:]).split():
        make_node(namespace, name, 'composite')
    parallel_ = make_node(dict(), 'parallel', 'composite')
    class parallel(parallel_):
        def __init__(self, fail, success, *children, **parameters):
            parameters['fail'] = fail
            parameters['success'] = success
            super(parallel, self).__init__(*children, **parameters)
    namespace['parallel'] = parallel


class Behavior(namedtuple('Behavior', [
    'name',
    'description',
    'scripts',
    'parameters',
    'root'])):
    def json(self):
        d = self._asdict()
        d['root'] = self.root.json()
        return d
    def to_string(self):
        return json.dumps(self.json(), indent=2)

if __name__ == '__main__':
    make_composites(globals())
    b = Behavior(
    'combat',
    'Fight enemies',
    ['/scripts/actions/clansman.lua'],
    {'play': True},
    sequence(
        selector(),
        parallel(1, -1,
            selector(),
            sequence(),
        )
    ))
    print(b.to_string())
