#!/usr/bin/env python

import behavior as bh

import json
import os
import re
from collections import OrderedDict

def load_nodes(namespace, validate=False):
    _nodes = {}
    for root in ['unpacked', 'diggle']:
        for head, dirs, files in os.walk('%s/behaviors/nodes' % root):
            for file in files:
                if file.endswith('.nodes'):
                    try:
                        nodes = json.load(open(os.path.join(head, file)), object_pairs_hook=OrderedDict)
                    except ValueError as e:
                        print('while parsing %s' % file)
                        raise e
                    for key, node in nodes.items():
                        symbol = node['name'].replace(' ', '_')
                        param_order=node.get('properties', None)
                        if param_order:
                            param_order = list(param_order.keys())
                        _nodes[symbol] = bh.make_node(namespace, node['name'], node['type'], param_order=param_order)
                        namespace[symbol] = _nodes[symbol]

        _FUNCTION_RE = re.compile(r'function ([a-zA-Z0-9\-]+)\(.*')
        for head, dirs, files in os.walk('%s/scripts/actions' % root):
            for file in files:
                if file.endswith('.lua'):
                    for line in open(os.path.join(head, file)):
                        m = _FUNCTION_RE.match(line)
                        if m:
                            name = m.group(1)
                            if name not in _nodes:
                                #print('%s not a node' % name)
                                continue
                            _nodes[name].script = os.path.join(head, file)
                            del _nodes[name]

    for k, node in _nodes.items():
        if node.ntype == 'action':
            pass
            #print('action %s does not have a script!' % k)

    bh.make_composites(namespace)
    def _f(self):
        if 'name' not in self.parameters and self.children:
            self.parameters['name'] = self.children[0].title
    namespace['logN'].prejson = _f

    namespace['selfk'] = {'key': 'self'}
    assert 'key' not in namespace
    namespace['key'] = lambda x: {'key': x}

if __name__ == '__main__':
    load_nodes(globals())
    b = bh.Behavior(
    'combat',
    'Fight enemies',
    ['/scripts/actions/clansman.lua'],
    {'play': True},
    sequence(
        cooldown(
            cooldown=2,
            onSuccess=True,  # mis-ordered
            onFail=False,
            child=canFire()
        ),
        parallel(1, -1, runner())
    ))
    print(b.to_string())

