#!/usr/bin/env python2

import load_nodes
import behavior

namespace = dict()
load_nodes.load_nodes(namespace)
namespace['Behavior'] = behavior.Behavior


def unparse_file(filename, output_filename=None):
    pass

def parse_file(filename, output_filename=None):
    if not output_filename:
        if filename.endswith('.pybehavior'):
            output_filename = filename.rpartition('.')[0] + '.behavior'
        else:
            raise Exception('Unrecognized filename %s' % filename)
    f = open(filename, 'r').read()
    b = eval(f, namespace)
    output = open(output_filename, 'wb')
    scripts = calculate_scripts(b)
    b = behavior.Behavior(**dict(b._asdict(), scripts=scripts))
    s = b.to_string()
    s = json_formatter.collapse_short_json2(s)
    output.write(s)


def calculate_scripts(behavior):
    scripts = set()
    frontier = [behavior.root]
    while frontier:
        top = frontier.pop()
        if top.script:
            script = '/' + top.script.partition('/')[-1]
            script = script.replace('\\', '/')
            scripts.add(script)
        for child in top.children:
            frontier.append(child)
    return list(sorted(scripts))


if __name__ == '__main__':
    import argparse
    import os

    parser = argparse.ArgumentParser()
    parser.add_argument('-r', '--recursive',
            default=False, action='store_const', const=True,
            help='Recursive into the given directory(s).')
    parser.add_argument('-o', '--output', default=None,
            help='Output filename, only combatible when parsing a single file.')
    parser.add_argument('filepaths', nargs='+',
            help='List of paths to files to compile.')
    args = parser.parse_args()

    if len(args.filepaths) >= 1 and args.output:
        raise Exception('Incompatible --output with multiple inputs')
    for path in args.filepaths:
        if args.recursive:
            for head, dirs, files in os.walk(path):
                for filename in files:
                    path = os.path.join(head, filename)
                    if filename.endswith('.pybehavior'):
                        print(path)
                        parse_file(path)
        else:
            parse_file(path, args.output)

