require "/scripts/pathutil.lua"

-- ACTIONS

-- output pos
function nearestPaintedBlock(args, board, node, dt)
  if args.position == nil then return false end
  if args.color == nil then return false end
  if args.maxRadius == nil then return false end
  local pos = nearestPaintedBlockFunc(
    args.position,
    args.color,
    args.maxRadius
    )
  if pos then
    return true, {position=pos}
  else
    return false
  end
end

-- output pos
function positionNearBlock(args, board, node, dt)
  if args.selfPosition == nil then return false end
  if args.blockPosition == nil then return false end
  if args.maxRadius == nil then return false end
  local x, y = table.unpack(args.blockPosition)
  local bounds = mcontroller.boundBox()
  local dx = x >= args.selfPosition[1] and -1 or 1
  for xo = 1,args.maxRadius do
    local pos = findGroundPosition(
        {x+dx*xo, y},
        -args.maxRadius,
        args.maxRadius,
        false, 
        {"Null", "Block"},
        bounds)
    if pos ~= nil then
      --world.damageTiles({args.blockPosition}, 'foreground', args.selfPosition, 'blockish', 2.0)
      return true, {position=pos}
    end
  end
  return false
end

function directionToPosition(args, board, node, dt)
  if args.origin == nil then return false end
  if args.position == nil then return false end
  local dir = args.position[1] - args.origin[1]
  dir = dir>0 and 1 or dir<0 and -1 or 0
  if dir == 0 then
    return false
  end
  return true, {direction=dir}
end

function mineBlock(args, board, node, dt)
  if args.sourcePosition == nil then return false end
  if args.position == nil then return false end
  if args.damageAmount == nil then return false end
  local ret = world.damageTiles({args.position}, 'foreground', args.sourcePosition, 'blockish', args.damageAmount)
  if world.material(args.position, 'foreground') == false then
    world.setMaterialColor(args.position, 'foreground', 0)
    world.setMaterialColor(args.position, 'background', 0)
  end
  return ret
end

function blockExists(args, board, node, dt)
  if args.position == nil then return false end
  return world.material(args.position, 'foreground') ~= false
end


-- NON-ACTIONS

function nearestPaintedBlockFunc(searchStartPos, matchColor, maxRadius)
  co = coroutine.create(spiralIter)
  local ran, dx, dy = coroutine.resume(co, maxRadius)
  local i = 0
  while dx do
    local pos = {searchStartPos[1] + dx, searchStartPos[2] + dy}
    local color = world.materialColor(pos, "foreground")
    if color == matchColor then
       sb.logWarn("found color " .. tostring(dx) .. ", " .. tostring(dy))
       return pos
    end
    ran, dx, dy = coroutine.resume(co)
    if i > 25 then 
      coroutine.yield()
      i = 0
    end
  end
end

-- SO 398299
function spiralIter(radius)
  local x, y = 0, 0
  local dx, dy = 0, -1
  for i=0,radius*radius do
    coroutine.yield(x, y)
    if x == y or (x < 0 and x == -y) or (x > 0 and x==1-y) then
      dx, dy = -dy, dx
    end
    x, y = x+dx, y+dy
  end
end

function dumpx(x)
  if type(x) == 'table' then
    local s = '{ '
    for k,v in pairs(x) do
      if type(k) ~= 'number' then k = '"'..k..'"' end
      s = s .. '['..k..'] = ' .. dumpx(v) .. ','
    end
    return s .. '} '
  else
    return tostring(x)
  end
end
