require "/scripts/pathutil.lua"

-- ACTIONS
function nearestFoodChest(args, board, node, dt)
  if args.position == nil then return false end
  if args.radius == nil then return false end
  local queryArgs = {
    name = "barrel",
    order = "nearest"
  }
  local containers = world.objectQuery(args.position, args.radius, queryArgs)
  for _, entityId in ipairs(containers) do
    items = world.containerItems(entityId)
    for index, item in pairs(items) do
      local foodValue = root.itemConfig(item).config.foodValue
      if foodValue then
        sb.logInfo("found " .. dumpx(item))
        return true, {chest = entityId}
      end
    end
  end
  return false
end

function eatFromFoodChest(args, board, node, dt)
  if args.chest == nil then return false end
  local items = world.containerItems(args.chest)
  if items == nil then
    sb.logWarn("tried to eat from non-chest")
    return false
  end
  for index, item in pairs(items) do
    local foodValue = root.itemConfig(item).config.foodValue
    if foodValue then
      status.modifyResource("food", foodValue)
      world.containerConsumeAt(args.chest, index-1, 1)
      -- TODO: Play sound
      --animator.playSound("/sfc/humanoid/eat3.ogg", world.entityPosition(args.chest))
      npc.dance("drink")
      return true
    end
  end
  return false
end

--------------------------------------------------

function dumpx(x)
  if type(x) == 'table' then
    local s = '{ '
    for k,v in pairs(x) do
      if type(k) ~= 'number' then k = '"'..k..'"' end
      s = s .. '['..k..'] = ' .. dumpx(v) .. ','
    end
    return s .. '} '
  else
    return tostring(x)
  end
end
